@@ -2,9 +2,12 @@
 // Name:        src/html/m_links.cpp
 // Purpose:     wxHtml module for links & anchors
 // Author:      Vaclav Slavik
-// RCS-ID:      $Id: nslinks.cpp,v 1.1 2007/02/24 18:57:09 kichik Exp $
+// RCS-ID:      $Id: nslinks.cpp,v 1.2 2007/03/08 01:47:14 pabs3 Exp $
 // Copyright:   (c) 1999 Vaclav Slavik
 // Licence:     wxWindows licence
+//
+// Reviewed for Unicode support by Jim Park -- 08/23/2007
+// Basically, compiling wxWidgets as Unicode should do it.
 /////////////////////////////////////////////////////////////////////////////
 
 #include "wx/wxprec.h"
@@ -13,7 +16,7 @@
     #pragma hdrstop
 #endif
 
-#if wxUSE_HTML && wxUSE_STREAMS
+#if wxUSE_HTML && wxUSE_STREAMS && !defined(NSISMENU_NOLINKTAGHANDLER)
 
 #ifndef WXPRECOMP
 #endif
@@ -24,8 +27,14 @@
 #include <wx/filefn.h>
 #include <wx/stdpaths.h>
 
+#include <nsis-sconf.h>
+
 FORCE_LINK_ME(nslinks)
 
+// TODO: These helper function declarations should be in a header file
+typedef enum { SUT_UNKNOWN = 0, SUT_BIN = 0x14, SUT_DOC = 0x24, SUT_WEB = 0x34 } SPECIALURLTYPE;
+SPECIALURLTYPE TransformUrl(wxString&Url);
+
 class wxHtmlAnchorCell : public wxHtmlCell
 {
 private:
@@ -53,28 +62,21 @@ TAG_HANDLER_BEGIN(A, "A")
         {
             wxHtmlLinkInfo oldlnk = m_WParser->GetLink();
             wxColour oldclr = m_WParser->GetActualColor();
-            wxString name(tag.GetParam( wxT("HREF") )), target;
+            wxString href(tag.GetParam( wxT("HREF") )), target;
 
             if (tag.HasParam( wxT("TARGET") )) target = tag.GetParam( wxT("TARGET") );
 
             wxColour colour = m_WParser->GetLinkColor();
-            wxHtmlLinkInfo linkInfo(name, target);
-
-            if (name.Left(3).IsSameAs((const wxChar*)"EX:", false))
+            wxHtmlLinkInfo linkInfo(href, target);
+            
+            wxString location = href;
+            SPECIALURLTYPE ut = TransformUrl(location);
+            if (ut == SUT_BIN || ut == SUT_DOC)
             {
-                wxString url = name.Mid(3);
-                if (!url.Left(7).IsSameAs((const wxChar*)"http://", false) && !url.Left(6).IsSameAs((const wxChar*)"irc://", false))
+                if (!wxFileExists(location) && !wxDirExists(location))
                 {
-                    wxString exePath = wxStandardPaths::Get().GetExecutablePath();
-                    wxString path = ::wxPathOnly(exePath);
-                    path.Append(wxFileName::GetPathSeparators()[0]);
-                    path.Append(url);
-
-                    if (!::wxFileExists(path) && !::wxDirExists(path))
-                    {
-                        colour = wxColour(0x80, 0x80, 0x80);
-                        linkInfo = wxHtmlLinkInfo("notinstalled.html", target);
-                    }
+                    colour = wxColour(0x80, 0x80, 0x80);
+                    linkInfo = wxHtmlLinkInfo(wxT("notinstalled.html"), target);
                 }
             }
 
@@ -98,12 +100,19 @@ TAG_HANDLER_BEGIN(A, "A")
 TAG_HANDLER_END(A)
 
 
-
+#if wxCHECK_VERSION(3, 0, 0)
+TAGS_MODULE_BEGIN(Links)
+#else
 TAGS_MODULE_BEGIN(CustomLinks)
+#endif
 
     TAGS_MODULE_ADD(A)
 
+#if wxCHECK_VERSION(3, 0, 0)
+TAGS_MODULE_END(Links)
+#else
 TAGS_MODULE_END(CustomLinks)
+#endif
 
 
 #endif
