@@ -17,6 +17,8 @@
   2. Altered source versions must be plainly marked as such, and must not be
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
+
+  Unicode support by Jim Park -- 08/24/2007
 */
 #include <windows.h>
 #include <stdio.h>
@@ -25,9 +27,10 @@
 #include "netinc.h"
 #include "util.h"
 #include "httpget.h"
-#include "../ExDLL/exdll.h"
 
-void *operator new( unsigned int num_bytes )
+#include <nsis/pluginapi.h> // nsis plugin
+
+void *operator new( size_t num_bytes )
 {
   return GlobalAlloc(GPTR,num_bytes);
 }
@@ -38,7 +41,7 @@ HMODULE     hModule;
 HWND        g_hwndProgressBar;
 HWND        g_hwndStatic;
 static int  g_cancelled;
-static void *lpWndProcOld;
+static WNDPROC lpWndProcOld;
 
 static UINT uMsgCreate;
 
@@ -55,7 +58,7 @@ static LRESULT CALLBACK ParentWndProc(HWND hwnd, UINT message, WPARAM wParam, LP
 
     if (wParam)
     {
-      childwnd = FindWindowEx((HWND) lParam, NULL, "#32770", NULL);
+      childwnd = FindWindowEx((HWND) lParam, NULL, _T("#32770"), NULL);
       hwndL = GetDlgItem(childwnd, 1016);
       hwndB = GetDlgItem(childwnd, 1027);
       HWND hwndP = GetDlgItem(childwnd, 1004);
@@ -83,8 +86,8 @@ static LRESULT CALLBACK ParentWndProc(HWND hwnd, UINT message, WPARAM wParam, LP
         GetWindowRect(hwndS, &ctlRect);
 
         HWND s = g_hwndStatic = CreateWindow(
-          "STATIC",
-          "",
+          _T("STATIC"),
+          _T(""),
           WS_CHILD | WS_CLIPSIBLINGS | SS_CENTER,
           0,
           wndRect.bottom / 2 - (ctlRect.bottom - ctlRect.top) / 2,
@@ -97,13 +100,13 @@ static LRESULT CALLBACK ParentWndProc(HWND hwnd, UINT message, WPARAM wParam, LP
         );
 
         DWORD dwStyle = WS_CHILD | WS_CLIPSIBLINGS;
-        dwStyle |= GetWindowLong(hwndP, GWL_STYLE) & PBS_SMOOTH;
+        dwStyle |= GetWindowLongPtr(hwndP, GWL_STYLE) & PBS_SMOOTH;
 
         GetWindowRect(hwndP, &ctlRect);
 
         HWND pb = g_hwndProgressBar = CreateWindow(
-          "msctls_progress32",
-          "",
+          _T("msctls_progress32"),
+          _T(""),
           dwStyle,
           0,
           wndRect.bottom / 2 + (ctlRect.bottom - ctlRect.top) / 2,
@@ -115,9 +118,7 @@ static LRESULT CALLBACK ParentWndProc(HWND hwnd, UINT message, WPARAM wParam, LP
           NULL
         );
 
-        long c;
-
-        c = SendMessage(hwndP, PBM_SETBARCOLOR, 0, 0);
+        LRESULT c = SendMessage(hwndP, PBM_SETBARCOLOR, 0, 0);
         SendMessage(hwndP, PBM_SETBARCOLOR, 0, c);
         SendMessage(pb, PBM_SETBARCOLOR, 0, c);
 
@@ -126,7 +127,7 @@ static LRESULT CALLBACK ParentWndProc(HWND hwnd, UINT message, WPARAM wParam, LP
         SendMessage(pb, PBM_SETBKCOLOR, 0, c);
 
         // set font
-        long hFont = SendMessage((HWND) lParam, WM_GETFONT, 0, 0);
+        LRESULT hFont = SendMessage((HWND) lParam, WM_GETFONT, 0, 0);
         SendMessage(pb, WM_SETFONT, hFont, 0);
         SendMessage(s, WM_SETFONT, hFont, 0);
 
@@ -179,13 +180,7 @@ static LRESULT CALLBACK ParentWndProc(HWND hwnd, UINT message, WPARAM wParam, LP
   }
   else
   {
-    return CallWindowProc(
-      (WNDPROC) lpWndProcOld,
-      hwnd,
-      message,
-      wParam,
-      lParam
-    );
+    return CallWindowProc(lpWndProcOld, hwnd, message, wParam, lParam);
   }
   return 0;
 }
@@ -196,7 +191,9 @@ extern "C" BOOL APIENTRY DllMain(HINSTANCE _hModule, DWORD  ul_reason_for_call,
   return TRUE;
 }
 
+#ifndef INT32_MAX
 #define INT32_MAX 0x7fffffff
+#endif
 
 int MulDiv64(int nNumber, __int64 nNumerator, __int64 nDenominator)
 {
@@ -213,7 +210,7 @@ int MulDiv64(int nNumber, __int64 nNumerator, __int64 nDenominator)
 
 static __int64 g_file_size;
 static DWORD g_dwLastTick = 0;
-void progress_callback(char *msg, __int64 read_bytes)
+void progress_callback(const char *msg, __int64 read_bytes)
 {
   // flicker reduction by A. Schiffler
   DWORD dwLastTick = g_dwLastTick;
@@ -222,7 +219,7 @@ void progress_callback(char *msg, __int64 read_bytes)
   {
     if (dwThisTick - dwLastTick > 500)
     {
-      SetWindowText(g_hwndStatic, msg);
+      SetWindowTextA(g_hwndStatic, msg);
       dwLastTick = dwThisTick;
     }
     if (g_file_size)
@@ -231,7 +228,7 @@ void progress_callback(char *msg, __int64 read_bytes)
   }
 }
 
-extern char *_strstr(char *i, char *s);
+extern char *_strstr(const char *i, const char *s);
 #define strstr _strstr
 
 extern "C"
@@ -239,12 +236,12 @@ extern "C"
 
 __declspec(dllexport) void download (HWND   parent,
               int    string_size,
-              char   *variables,
+              TCHAR   *variables,
               stack_t **stacktop)
 {
   char buf[1024];
   char url[1024];
-  char filename[1024];
+  TCHAR filenameT[1024];
   static char proxy[1024];
   BOOL bSuccess=FALSE;
   int timeout_ms=30000;
@@ -252,7 +249,7 @@ __declspec(dllexport) void download (HWND   parent,
   int manualproxy=0;
   int translation_version;
 
-  char *error=NULL;
+  const char *error=NULL;
 
   // translation version 2 & 1
   static char szDownloading[1024]; // "Downloading %s"
@@ -277,83 +274,92 @@ __declspec(dllexport) void download (HWND   parent,
 
   EXDLL_INIT();
 
-  popstring(url);
-  if (!lstrcmpi(url, "/TRANSLATE2")) {
-    popstring(szDownloading);
-    popstring(szConnecting);
-    popstring(szSecond);
-    popstring(szMinute);
-    popstring(szHour);
-    popstring(szSeconds);
-    popstring(szMinutes);
-    popstring(szHours);
-    popstring(szProgress);
-    popstring(url);
+  PopStringA(url);
+  if (!lstrcmpiA(url, "/TRANSLATE2")) {
+    PopStringA(szDownloading);
+    PopStringA(szConnecting);
+    PopStringA(szSecond);
+    PopStringA(szMinute);
+    PopStringA(szHour);
+    PopStringA(szSeconds);
+    PopStringA(szMinutes);
+    PopStringA(szHours);
+    PopStringA(szProgress);
+    PopStringA(url);
     translation_version=2;
-  } else if (!lstrcmpi(url, "/TRANSLATE")) {
-    popstring(szDownloading);
-    popstring(szConnecting);
-    popstring(szSecond);
-    popstring(szMinute);
-    popstring(szHour);
-    popstring(szPlural);
-    popstring(szProgress);
-    popstring(szRemaining);
-    popstring(url);
+  } else if (!lstrcmpiA(url, "/TRANSLATE")) {
+    PopStringA(szDownloading);
+    PopStringA(szConnecting);
+    PopStringA(szSecond);
+    PopStringA(szMinute);
+    PopStringA(szHour);
+    PopStringA(szPlural);
+    PopStringA(szProgress);
+    PopStringA(szRemaining);
+    PopStringA(url);
     translation_version=1;
   } else {
-    lstrcpy(szDownloading, "Downloading %s");
-    lstrcpy(szConnecting, "Connecting ...");
-    lstrcpy(szSecond, " (1 second remaining)");
-    lstrcpy(szMinute, " (1 minute remaining)");
-    lstrcpy(szHour, " (1 hour remaining)");
-    lstrcpy(szSeconds, " (%u seconds remaining)");
-    lstrcpy(szMinutes, " (%u minutes remaining)");
-    lstrcpy(szHours, " (%u hours remaining)");
-    lstrcpy(szProgress, "%skB (%d%%) of %skB at %u.%01ukB/s");
+    lstrcpyA(szDownloading, "Downloading %s");
+    lstrcpyA(szConnecting, "Connecting ...");
+    lstrcpyA(szSecond, " (1 second remaining)");
+    lstrcpyA(szMinute, " (1 minute remaining)");
+    lstrcpyA(szHour, " (1 hour remaining)");
+    lstrcpyA(szSeconds, " (%u seconds remaining)");
+    lstrcpyA(szMinutes, " (%u minutes remaining)");
+    lstrcpyA(szHours, " (%u hours remaining)");
+    lstrcpyA(szProgress, "%skB (%d%%) of %skB at %u.%01ukB/s");
     translation_version=2;
   }
-  lstrcpyn(buf, url, 10);
-  if (!lstrcmpi(buf, "/TIMEOUT=")) {
+  lstrcpynA(buf, url, 10);
+  if (!lstrcmpiA(buf, "/TIMEOUT=")) {
     timeout_ms=my_atoi(url+9);
-    popstring(url);
+    PopStringA(url);
   }
-  if (!lstrcmpi(url, "/PROXY")) {
+  if (!lstrcmpiA(url, "/PROXY")) {
     getieproxy=0;
     manualproxy=1;
-    popstring(proxy);
-    popstring(url);
+    PopStringA(proxy);
+    PopStringA(url);
   }
-  if (!lstrcmpi(url, "/NOIEPROXY")) {
+  if (!lstrcmpiA(url, "/NOIEPROXY")) {
     getieproxy=0;
-    popstring(url);
+    PopStringA(url);
   }
-  popstring(filename);
+  popstring(filenameT);
 
-  HANDLE hFile = CreateFile(filename,GENERIC_WRITE,FILE_SHARE_READ,NULL,CREATE_ALWAYS,0,NULL);
+  static char main_buf[8192];
+  char *filenameA;
+#ifdef _UNICODE
+  filenameA = main_buf;
+  wsprintfA(filenameA, "%S", filenameT);
+#else
+  filenameA = filenameT;
+#endif
+
+  HANDLE hFile = CreateFile(filenameT,GENERIC_WRITE,FILE_SHARE_READ,NULL,CREATE_ALWAYS,0,NULL);
 
   if (hFile == INVALID_HANDLE_VALUE)
   {
-    wsprintf(buf, "Unable to open %s", filename);
+    wsprintfA(buf, "Unable to open %s", filenameA);
     error = buf;
   }
   else
   {
     if (parent)
     {
-      uMsgCreate = RegisterWindowMessage("nsisdl create");
+      uMsgCreate = RegisterWindowMessage(_T("nsisdl create"));
 
-      lpWndProcOld = (void *)SetWindowLong(parent,GWL_WNDPROC,(long)ParentWndProc);
+      lpWndProcOld = (WNDPROC)SetWindowLongPtr(parent,GWLP_WNDPROC,(LONG_PTR)ParentWndProc);
 
       SendMessage(parent, uMsgCreate, TRUE, (LPARAM) parent);
 
       // set initial text
-      char *p = filename;
+      char *p = filenameA;
       while (*p) p++;
-      while (*p != '\\' && p != filename) p = CharPrev(filename, p);
-      wsprintf(buf, szDownloading, p != filename ? p + 1 : p);
-      SetDlgItemText(childwnd, 1006, buf);
-      SetWindowText(g_hwndStatic, szConnecting);
+      while (*p !='\\' && p != filenameA) p = CharPrevA(filenameA, p);
+      wsprintfA(buf, szDownloading, p != filenameA ? p + 1 : p);
+      SetDlgItemTextA(childwnd, 1006, buf);
+      SetWindowTextA(g_hwndStatic, szConnecting);
     }
     {
       WSADATA wsaData;
@@ -361,20 +367,18 @@ __declspec(dllexport) void download (HWND   parent,
 
       JNL_HTTPGet *get = 0;
 
-      static char main_buf[8192];
-      char *buf=main_buf;
-      char *p=NULL;
+      char *buf = main_buf, *p = NULL;
 
       HKEY hKey;
-      if (getieproxy && RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",0,KEY_READ,&hKey) == ERROR_SUCCESS)
+      if (getieproxy && RegOpenKeyExA(HKEY_CURRENT_USER,"Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",0,KEY_READ,&hKey) == ERROR_SUCCESS)
       {
         DWORD l = 4;
         DWORD t;
         DWORD v;
-        if (RegQueryValueEx(hKey,"ProxyEnable",NULL,&t,(unsigned char *)&v,&l) == ERROR_SUCCESS && t == REG_DWORD && v)
+        if (RegQueryValueExA(hKey,"ProxyEnable",NULL,&t,(unsigned char*)&v,&l) == ERROR_SUCCESS && t == REG_DWORD && v)
         {
           l=8192;
-          if (RegQueryValueEx(hKey,"ProxyServer",NULL,&t,(unsigned char *)buf,&l ) == ERROR_SUCCESS && t == REG_SZ)
+          if (RegQueryValueExA(hKey,"ProxyServer",NULL,&t,(unsigned char *)buf,&l ) == ERROR_SUCCESS && t == REG_SZ)
           {
             p=strstr(buf,"http=");
             if (!p) p=buf;
@@ -417,7 +421,7 @@ __declspec(dllexport) void download (HWND   parent,
           if (parent)
           {
             SendMessage(parent, uMsgCreate, FALSE, (LPARAM) parent);
-            SetWindowLong(parent, GWL_WNDPROC, (long)lpWndProcOld);
+            SetWindowLongPtr(parent, GWLP_WNDPROC, (LONG_PTR)lpWndProcOld);
           }
           break;
         }
@@ -425,7 +429,7 @@ __declspec(dllexport) void download (HWND   parent,
         st = get->run ();
 
         if (st == -1) {
-          lstrcpyn(url, get->geterrorstr(), sizeof(url));
+          lstrcpynA(url, get->geterrorstr(), sizeof(url));
           error = url;
         } else if (st == 1) {
           if (sofar < cl || get->get_status () != 2)
@@ -500,14 +504,14 @@ __declspec(dllexport) void download (HWND   parent,
                   myitoa64(sofar/1024, sofar_str);
                   myitoa64(cl/1024, cl_str);
 
-                  wsprintf (buf,
+                  wsprintfA (buf,
                         szProgress, //%skB (%d%%) of %skB @ %u.%01ukB/s
                         sofar_str,
                         MulDiv64(100, sofar, cl),
                         cl_str,
                         bps/1024,((bps*10)/1024)%10
                         );
-                  if (remain) wsprintf(buf+lstrlen(buf),rtext,
+                  if (remain) wsprintfA(buf+lstrlenA(buf),rtext,
                         remain
                         );
                 } else if (translation_version == 1) {
@@ -523,14 +527,14 @@ __declspec(dllexport) void download (HWND   parent,
                     }
                   }
 
-                  wsprintf (buf,
+                  wsprintfA (buf,
                         szProgress, //%dkB (%d%%) of %dkB @ %d.%01dkB/s
                         int(sofar/1024),
                         MulDiv64(100, sofar, cl),
                         int(cl/1024),
                         bps/1024,((bps*10)/1024)%10
                         );
-                  if (remain) wsprintf(buf+lstrlen(buf),szRemaining,
+                  if (remain) wsprintfA(buf+lstrlenA(buf),szRemaining,
                         remain,
                         rtext,
                         remain==1?"":szPlural
@@ -577,16 +581,16 @@ __declspec(dllexport) void download (HWND   parent,
   }
 
   if (g_cancelled || !bSuccess) {
-    DeleteFile(filename);
+    DeleteFile(filenameT);
   }
 
-  pushstring(error);
+  PushStringA(error);
 }
 
 
 __declspec(dllexport) void download_quiet(HWND   parent,
               int    stringsize,
-              char   *variables,
+              TCHAR   *variables,
               stack_t **stacktop)
 {
   g_hwndProgressBar=0;
