@@ -1,3 +1,5 @@
+// Unicode support by Jim Park -- 08/23/2007
+
 #include "stdafx.h"
 #include "Plugin.h"
 #include "System.h"
@@ -7,46 +9,57 @@ typedef struct tagTempStack TempStack;
 struct tagTempStack
 {
     TempStack *Next;
-    char Data[0];
+    TCHAR Data[0];
 };
 TempStack *tempstack = NULL;
 
-PLUGINFUNCTIONSHORT(Alloc)
+static void AllocWorker(unsigned int mult)
 {
-    int size;
-    if ((size = popint()) == 0)
+    size_t size;
+    if ((size = popintptr()) == 0)
     {
-        pushint(0);
+        system_pushint(0);
         return;
     }
-    pushint((int) GlobalAlloc(GPTR, size));
+    system_pushintptr((INT_PTR) GlobalAlloc(GPTR, size * mult));
+}
+
+PLUGINFUNCTIONSHORT(Alloc)
+{
+    AllocWorker(sizeof(unsigned char));
+}
+PLUGINFUNCTIONEND
+
+PLUGINFUNCTIONSHORT(StrAlloc)
+{
+    AllocWorker(sizeof(TCHAR));
 }
 PLUGINFUNCTIONEND
 
 PLUGINFUNCTIONSHORT(Copy)
 {
-    int size = 0;
+    SIZE_T size = 0;
     HANDLE source, dest;
-    char *str;
+    TCHAR *str;
     // Get the string
-    if ((str = popstring()) == NULL) return;
+    if ((str = system_popstring()) == NULL) return;
 
     // Check for size option
-    if (str[0] == '/')
+    if (str[0] == _T('/'))
     {
-        size = (int) myatoi(str+1);
-        dest = (HANDLE) popint();
+        size = (SIZE_T) StrToIntPtr(str+1);
+        dest = (HANDLE) popintptr();
     }
-    else dest = (HANDLE) myatoi(str);
-    source = (HANDLE) popint();
+    else dest = (HANDLE) StrToIntPtr(str);
+    source = (HANDLE) popintptr();
 
     // Ok, check the size
-    if (size == 0) size = (int) GlobalSize(source);
+    if (size == 0) size = (SIZE_T) GlobalSize(source);
     // and the destinantion
-    if ((int) dest == 0) 
+    if (!dest) 
     {
         dest = GlobalAlloc((GPTR), size);
-        pushint((int) dest);
+        system_pushintptr((INT_PTR) dest);
     }
 
     // COPY!
@@ -56,24 +69,18 @@ PLUGINFUNCTIONSHORT(Copy)
 }
 PLUGINFUNCTIONEND
 
-PLUGINFUNCTIONSHORT(Free)
-{
-    GlobalFree((HANDLE) popint());
-}
-PLUGINFUNCTIONEND
-
 PLUGINFUNCTION(Store)
 {
     TempStack *tmp;
-    int size = ((INST_R9+1)*g_stringsize);    
+    int size = ((INST_R9+1)*g_stringsize*sizeof(TCHAR));
 
-    char *command, *cmd = command = popstring();
+    TCHAR *command, *cmd = command = system_popstring();
     while (*cmd != 0)
     {
         switch (*(cmd++))
         {
-        case 's':
-        case 'S':
+        case _T('s'):
+        case _T('S'):
             // Store the whole variables range
             tmp = (TempStack*) GlobalAlloc(GPTR, sizeof(TempStack)+size);
             tmp->Next = tempstack;
@@ -82,8 +89,8 @@ PLUGINFUNCTION(Store)
             // Fill with data
             copymem(tempstack->Data, g_variables, size);
             break;
-        case 'l':
-        case 'L':
+        case _T('l'):
+        case _T('L'):
             if (tempstack == NULL) break;
 
             // Fill with data
@@ -94,15 +101,15 @@ PLUGINFUNCTION(Store)
             GlobalFree((HANDLE) tempstack);
             tempstack = tmp;
             break;
-        case 'P':
+        case _T('P'):
             *cmd += 10;
-        case 'p':
-            GlobalFree((HANDLE) pushstring(getuservariable(*(cmd++)-'0')));
+        case _T('p'):
+            GlobalFree((HANDLE) system_pushstring(system_getuservariable(*(cmd++)-_T('0'))));
             break;
-        case 'R':
+        case _T('R'):
             *cmd += 10;
-        case 'r':
-            GlobalFree((HANDLE) setuservariable(*(cmd++)-'0', popstring()));
+        case _T('r'):
+            GlobalFree((HANDLE) system_setuservariable(*(cmd++)-_T('0'), system_popstring()));
             break;
         }
     }
