@@ -1,8 +1,16 @@
+// Unicode support by Jim Park -- 08/10/2007
+
 #include <windows.h>
 #include <shlobj.h>
 
+#include <nsis/pluginapi.h> // nsis plugin
+#include <nsis/nsis_tchar.h>
+
 #include "defs.h"
-#include "nsis.h"
+
+#ifndef COUNTOF
+#define COUNTOF(a) (sizeof(a)/sizeof(a[0]))
+#endif
 
 int CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lp, LPARAM pData) {
   if (uMsg == BFFM_INITIALIZED)
@@ -11,26 +19,26 @@ int CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lp, LPARAM pData) {
   return 0;
 }
 
-void __declspec(dllexport) SelectFolderDialog(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) SelectFolderDialog(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   BROWSEINFO bi;
 
-  char result[MAX_PATH];
-  char initial[MAX_PATH];
-  char title[1024];
+  TCHAR result[MAX_PATH];
+  TCHAR initial[MAX_PATH];
+  TCHAR title[1024];
   LPITEMIDLIST resultPIDL;
 
   EXDLL_INIT();
 
-  if (popstring(title, sizeof(initial)))
+  if (popstringn(title, COUNTOF(initial)))
   {
-    pushstring("error");
+    pushstring(_T("error"));
     return;
   }
 
-  if (popstring(initial, sizeof(title)))
+  if (popstringn(initial, COUNTOF(title)))
   {
-    pushstring("error");
+    pushstring(_T("error"));
     return;
   }
 
@@ -63,7 +71,7 @@ void __declspec(dllexport) SelectFolderDialog(HWND hwndParent, int string_size,
   resultPIDL = SHBrowseForFolder(&bi);
   if (!resultPIDL)
   {
-    pushstring("error");
+    pushstring(_T("error"));
     return;
   }
 
@@ -73,20 +81,22 @@ void __declspec(dllexport) SelectFolderDialog(HWND hwndParent, int string_size,
   }
   else
   {
-    pushstring("error");
+    pushstring(_T("error"));
   }
 
   CoTaskMemFree(resultPIDL);
 }
 
-void __declspec(dllexport) SelectFileDialog(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) SelectFileDialog(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   OPENFILENAME ofn={0,}; // XXX WTF
   int save;
-  char type[5];
-  char path[1024];
-  char filter[1024];
-  char currentDirectory[1024];
+  TCHAR type[5];
+  static TCHAR path[1024];
+  static TCHAR filter[1024];
+  static TCHAR currentDirectory[1024];
+  static TCHAR initialDir[1024];
+  DWORD gfa;
 
   EXDLL_INIT();
 
@@ -94,28 +104,38 @@ void __declspec(dllexport) SelectFileDialog(HWND hwndParent, int string_size, ch
   ofn.hwndOwner = hwndParent;
   ofn.lpstrFilter = filter;
   ofn.lpstrFile = path;
-  ofn.nMaxFile  = sizeof(path);
+  ofn.nMaxFile  = COUNTOF(path);
   //ofn.Flags = pField->nFlags & (OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_CREATEPROMPT | OFN_EXPLORER);
   ofn.Flags = OFN_CREATEPROMPT | OFN_EXPLORER;
 
-  popstring(type, sizeof(type));
-  popstring(path, sizeof(path));
-  popstring(filter, sizeof(filter));
+  popstringn(type, COUNTOF(type));
+  popstringn(path, COUNTOF(path));
+  popstringn(filter, COUNTOF(filter));
 
-  save = !lstrcmpi(type, "save");
+  save = !lstrcmpi(type, _T("save"));
+
+  // Check if the path given is a folder. If it is we initialize the 
+  // ofn.lpstrInitialDir parameter
+  gfa = GetFileAttributes(path);
+  if ((gfa != INVALID_FILE_ATTRIBUTES) && (gfa & FILE_ATTRIBUTE_DIRECTORY))
+  {
+    lstrcpy(initialDir, path);
+    ofn.lpstrInitialDir = initialDir;
+    path[0] = _T('\0'); // disable initial file selection as path is actually a directory
+  }
 
   if (!filter[0])
   {
-    lstrcpy(filter, "All Files|*.*");
+    lstrcpy(filter, _T("All Files|*.*"));
   }
 
   {
     // Convert the filter to the format required by Windows: NULL after each
     // item followed by a terminating NULL
-    char *p = filter;
+    TCHAR *p = filter;
     while (*p) // XXX take care for 1024
     {
-      if (*p == '|')
+      if (*p == _T('|'))
       {
         *p++ = 0;
       }
@@ -128,7 +148,7 @@ void __declspec(dllexport) SelectFileDialog(HWND hwndParent, int string_size, ch
     *p = 0;
   }
 
-  GetCurrentDirectory(sizeof(currentDirectory), currentDirectory); // save working dir
+  GetCurrentDirectory(COUNTOF(currentDirectory), currentDirectory); // save working dir
 
   if ((save ? GetSaveFileName(&ofn) : GetOpenFileName(&ofn)))
   {
@@ -136,22 +156,22 @@ void __declspec(dllexport) SelectFileDialog(HWND hwndParent, int string_size, ch
   }
   else if (CommDlgExtendedError() == FNERR_INVALIDFILENAME)
   {
-    *path = '\0';
+    *path = _T('\0');
     if ((save ? GetSaveFileName(&ofn) : GetOpenFileName(&ofn)))
     {
       pushstring(path);
     }
     else
     {
-      pushstring("");
+      pushstring(_T(""));
     }
   }
   else
   {
-    pushstring("");
+    pushstring(_T(""));
   }
 
   // restore working dir
   // OFN_NOCHANGEDIR doesn't always work (see MSDN)
   SetCurrentDirectory(currentDirectory);
-}
\ No newline at end of file
+}
