@@ -3,6 +3,7 @@ requires Python Image Library - http://www.pythonware.com/products/pil/
 requires grep and diff - http://www.mingw.org/msys.shtml
 requires command line svn - http://subversion.tigris.org/
 requires pysvn - http://pysvn.tigris.org/
+requires win32com - http://starship.python.net/~skippy/win32/Downloads.html
 
 example release.cfg:
 =========================
@@ -19,7 +20,7 @@ VER_BUILD=0
 
 [svn]
 SVN="C:\svn-win32\bin\svn.exe"
-SVNROOT=https://nsis.svn.sourceforge.net/svnroot/nsis/NSIS
+SVNROOT=https://svn.code.sf.net/p/nsis/code/NSIS
 
 [compression]
 TAR_BZ2=7zatarbz2.bat %s %s
@@ -28,11 +29,18 @@ ZIP="C:\Program Files\7-zip\7za.exe" a -tzip %s -mx9 -mfb=255 -mpass=4 %s
 [rsh]
 RSH="C:\Program Files\PuTTY\plink.exe" -2 -l kichik nsis.sourceforge.net
 
+[sftp]
+SFTP="C:\Program Files\PuTTY\psftp.exe" -2 -l kichik,nsis -batch -b %s frs.sourceforge.net
+
 [wiki]
+PURGE_URL=http://nsis.sourceforge.net/%s?action=purge
 UPDATE_URL=http://nsis.sourceforge.net/Special:Simpleupdate?action=raw
 
 [svn2cl]
-SVN2CL_XSL=svl2cl.xsl
+SVN2CL_XSL=svn2cl.xsl
+
+[options]
+SKIP_CPPUNIT=no
 =========================
 
 7zatarbz2.bat:
@@ -58,7 +66,6 @@ import sys
 import time
 import Image, ImageFont, ImageDraw
 from ConfigParser import ConfigParser
-from ftplib import FTP
 import time
 import pysvn
 
@@ -76,6 +83,8 @@ VER_MINOR = cfg.get('version', 'VER_MINOR')
 VER_REVISION = cfg.get('version', 'VER_REVISION')
 VER_BUILD = cfg.get('version', 'VER_BUILD')
 
+PRE_RELEASE_VERSION = 'a' in VERSION or 'b' in VERSION
+
 SVN = cfg.get('svn', 'SVN')
 SVNROOT = cfg.get('svn', 'SVNROOT')
 
@@ -84,11 +93,15 @@ ZIP = cfg.get('compression', 'ZIP')
 
 RSH = cfg.get('rsh', 'RSH')
 
+SFTP = cfg.get('sftp', 'SFTP')
+
 PURGE_URL = cfg.get('wiki', 'PURGE_URL')
 UPDATE_URL = cfg.get('wiki', 'UPDATE_URL')
 
 SVN2CL_XSL = cfg.get('svn2cl', 'SVN2CL_XSL')
 
+SKIP_CPPUINT = cfg.get('options', 'SKIP_CPPUNIT')
+
 ### config env
 
 SVN_TAG = 'v' + ''.join(VERSION.split('.'))
@@ -122,7 +135,7 @@ def run(command, log_level, err, wanted_ret = 0, log_dir = '.'):
 	else:
 		raise ValueError
 
-	ret = os.system(cmd)
+	ret = os.system('if 1==1 ' + cmd)
 
 	# sleep because for some weird reason, running cvs.exe hugs
 	# the release log for some time after os.system returns
@@ -153,51 +166,59 @@ def RunTests():
 	print 'running tests...'
 
 	run(
-		'scons -C .. test',
+		'scons -C .. %s' % (SKIP_CPPUINT == 'yes' and 'test-scripts' or 'test'),
 		LOG_ALL,
 		'tests failed - see test.log for details'
 	)
 
 def TestSubversionEOL():
-  print 'ensuring EOL...'
+	print 'ensuring EOL...'
 
-  from os import walk
-  from os.path import join
-  from os.path import splitext
-  
-  eoldict = {
-    '.nsh' : 'native',
-    '.nsi' : 'native',
-    '.txt' : 'native',
-    '.ini' : 'CRLF',
-    '.dsp' : 'CRLF',
-    '.dsw' : 'CRLF'
-  }
+	from os import walk
+	from os.path import join
+	from os.path import splitext
+	
+	eoldict = {
+		'.nsh' : 'native',
+		'.nsi' : 'native',
+		'.txt' : 'native',
+		'.ini' : 'CRLF',
+		'.dsp' : 'CRLF',
+		'.dsw' : 'CRLF'
+	}
 
-  exceptions = ['newfile.txt', 'oldfile.txt']
+	exceptions = ['newfile.txt', 'oldfile.txt']
 
-  svn = pysvn.Client()
+	svn = pysvn.Client()
 
-  for root, dirs, files in walk('..'):
-    if '.svn' not in dirs:
-      continue
+	for root, dirs, files in walk('..'):
+		def versioned(f):
+			try:
+				s = svn.status(join(root, f))[0].text_status
+			except pysvn.ClientError, e:
+				if 'not a working copy' in e.message:
+					return False
+				else:
+					raise
+			return s != pysvn.wc_status_kind.unversioned
 
-    def versioned(f):
-      s = svn.status(join(root, f))[0].text_status
-      return s != pysvn.wc_status_kind.unversioned
+		svn_files = filter(versioned, files)
+		svn_files = filter(lambda x: x not in exceptions, svn_files)
 
-    svn_files = filter(versioned, files)
-    svn_files = filter(lambda x: x not in exceptions, svn_files)
-
-    for f in svn_files:
-      ext = splitext(f)[1]
-      if ext in eoldict.keys():
-        eol = eoldict[ext]
-        s = svn.propget('svn:eol-style', join(root, f)).values()
-        if not s or s[0] != eol:
-          print '*** %s has bad eol-style' % f
-          log('*** %s has bad eol-style' % f)
-          exit()
+		bad_eol = False
+		for f in svn_files:
+			ext = splitext(f)[1]
+			if ext in eoldict.keys():
+				eol = eoldict[ext]
+				path = join(root, f)
+				s = svn.propget('svn:eol-style', path).values()
+				if not s or s[0] != eol:
+					print '*** %s has bad eol-style' % path
+					log('*** %s has bad eol-style' % path)
+					bad_eol = True
+					
+		if bad_eol:
+			exit()
 
 def CreateMenuImage():
 	print 'creating images...'
@@ -250,7 +271,7 @@ def TestInstaller():
 	)
 
 	run(
-		'diff -r insttest insttestscons | grep -v uninst-nsis.exe',
+		'diff -r insttest insttestscons | grep -v uninst-nsis.exe | grep -v NSIS.exe',
 		LOG_ALL,
 		'scons and installer installations differ',
 		1
@@ -284,7 +305,7 @@ def CreateChangeLog():
 
 		print 'downloading svn2cl.xsl stylesheet...'
 
-		urllib.urlretrieve('http://svn.collab.net/repos/svn/trunk/contrib/client-side/svn2cl/svn2cl.xsl', SVN2CL_XSL)
+		urllib.urlretrieve('http://svn.apache.org/repos/asf/subversion/trunk/contrib/client-side/svn2cl/svn2cl.xsl', SVN2CL_XSL)
 
 	print 'generating ChangeLog...'
 
@@ -362,32 +383,37 @@ def CreateSpecialBuilds():
 def UploadFiles():
 	print 'uploading files to SourceForge...'
 
-	def upload(ftp, file):
-		print '  uploading %s...' % file
-		ftp.storbinary('STOR /incoming/%s' % file.split('\\')[-1], open(file, 'rb'))
+	folder = 'NSIS 3/' + VERSION
+	if PRE_RELEASE_VERSION:
+		folder = 'NSIS 3 Pre-release/' + VERSION
 
-	ftp = FTP('upload.sourceforge.net')
-	ftp.login()
+	sftpcmds = file('sftp-commands', 'w')
+	sftpcmds.write('mkdir "/home/frs/project/nsis/%s"\n' % folder)
+	sftpcmds.write('cd "/home/frs/project/nsis/%s"\n' % folder)
+	sftpcmds.write('put %s.tar.bz2\n' % newverdir)
+	sftpcmds.write('put %s\\nsis-%s-setup.exe\n' % (newverdir, VERSION))
+	sftpcmds.write('put %s\\nsis-%s.zip\n' % (newverdir, VERSION))
+	sftpcmds.write('put nsis-%s-log.zip\n' % VERSION)
+	sftpcmds.write('put nsis-%s-strlen_8192.zip\n' % VERSION)
+	sftpcmds.write('put %s\\build\\urelease\\Docs\\chm\\SectionF.1.html RELEASE.html\n' % newverdir)
+	sftpcmds.close()
 
-	upload(ftp, newverdir + '.tar.bz2')
-	upload(ftp, newverdir + '\\nsis-%s-setup.exe' % VERSION)
-	upload(ftp, newverdir + '\\nsis-%s.zip' % VERSION)
-	upload(ftp, 'nsis-%s-log.zip' % VERSION)
-	upload(ftp, 'nsis-%s-strlen_8192.zip' % VERSION)
+	run(
+		SFTP % 'sftp-commands',
+		LOG_ERRORS,
+		'upload failed'
+	)
 
-	ftp.quit()
+	os.unlink('sftp-commands')
 
 def ManualRelease():
-	print 'release url:'
-	print '  http://sourceforge.net/project/admin/qrs.php?package_id=0&group_id=22049'
+	print 'go fix release notes...'
+	print '  http://nsis.sf.net/rn/new'
 	print
 
-	sys.stdout.write('What\'s the SF release id of the new version? ')
-	release_id = raw_input()
+	raw_input()
 
-	return release_id
-
-def UpdateWiki(release_id):
+def UpdateWiki():
 	print 'updating wiki...'
 
 	def update_wiki_page(page, data, summary):
@@ -401,31 +427,30 @@ def UpdateWiki(release_id):
 		post += '&su_data=' + urllib.quote(data)
 		post += '&su_summary=' + urllib.quote(summary)
 		
-		if urllib.urlopen(UPDATE_URL, post).read() != 'success':
+		if urllib.urlopen(UPDATE_URL, post).read().strip() != 'success':
 			log('*** failed updating `%s` wiki page' % page)
 			print '	*** failed updating `%s` wiki page' % page
 
-	update_wiki_page('Template:NSISVersion', VERSION, 'new version')
-	update_wiki_page('Template:NSISReleaseDate', time.strftime('%B %d, %Y'), 'new version')
-	update_wiki_page('Template:NSISReleaseID', release_id, 'new version')
+	if not PRE_RELEASE_VERSION:
+		update_wiki_page('Template:NSISVersion', VERSION, 'new version')
+		update_wiki_page('Template:NSISReleaseDate', time.strftime('%B %d, %Y'), 'new version')
+		update_wiki_page('Template:NSISIsPreRelease', 'no', 'new version')
+	else:
+		update_wiki_page('Template:NSISPreVersion', VERSION, 'new version')
+		update_wiki_page('Template:NSISPreReleaseDate', time.strftime('%B %d, %Y'), 'new version')
+		update_wiki_page('Template:NSISIsPreRelease', 'yes', 'new version')
 
-def UpdateChangeLog():
-	run(
-		'%s /home/groups/n/ns/nsis/bin/touch_cl_tag.sh' % RSH,
-		LOG_ALL,
-		'change log tag modification failed'
-	)
+	os.system('start ' + PURGE_URL % 'Download')
 
 def ToDo():
 	print 'automatic phase done\n'
 	print """
- * %s
  * Edit update.php
  * Post news item
  * http://en.wikipedia.org/w/index.php?title=Nullsoft_Scriptable_Install_System&action=edit
  * Update Freshmeat
  * Update BetaNews
-	""" % (PURGE_URL % 'Download')
+	"""
 
 def CloseLog():
 	log('done')
@@ -446,8 +471,7 @@ CreateSourceTarball()
 BuildRelease()
 CreateSpecialBuilds()
 UploadFiles()
-release_id = ManualRelease()
-UpdateWiki(release_id)
-UpdateChangeLog()
+ManualRelease()
+UpdateWiki()
 ToDo()
 CloseLog()
