@@ -1,24 +1,20 @@
 /*
-  Copyright (C) 2002 Amir Szekely <kichik@netvision.net.il>
-
-  This software is provided 'as-is', without any express or implied
-  warranty.  In no event will the authors be held liable for any damages
-  arising from the use of this software.
-
-  Permission is granted to anyone to use this software for any purpose,
-  including commercial applications, and to alter it and redistribute it
-  freely, subject to the following restrictions:
-
-  1. The origin of this software must not be misrepresented; you must not
-  claim that you wrote the original software. If you use this software
-  in a product, an acknowledgment in the product documentation would be
-  appreciated but is not required.
-
-  2. Altered source versions must be plainly marked as such, and must not be
-  misrepresented as being the original software.
-
-  3. This notice may not be removed or altered from any source distribution.
-*/
+ * DialogTemplate.h
+ * 
+ * This file is a part of NSIS.
+ * 
+ * Copyright (C) 2002-2019 Amir Szekely <kichik@netvision.net.il> and Contributors
+ * 
+ * Licensed under the zlib/libpng license (the "License");
+ * you may not use this file except in compliance with the License.
+ * 
+ * Licence details can be found in the file COPYING.
+ * 
+ * This software is provided 'as-is', without any express or implied
+ * warranty.
+ *
+ * Reviewed for Unicode support by Jim Park -- 08/21/2007
+ */
 
 #if !defined(AFX_DIALOGTEMPLATE_H__C5A973AF_0F56_4BEC_814A_79318E2EB4AC__INCLUDED_)
 #define AFX_DIALOGTEMPLATE_H__C5A973AF_0F56_4BEC_814A_79318E2EB4AC__INCLUDED_
@@ -28,6 +24,7 @@
 #endif // _MSC_VER > 1000
 
 #include "Platform.h"
+#include "winchar.h"
 
 #include <vector>
 #include <stdexcept>
@@ -49,8 +46,8 @@ struct DialogItemTemplate {
   DWORD  dwStyle;
   WORD   wId;
 
-  char  *szClass;
-  char  *szTitle;
+  WINWCHAR *szClass;
+  WINWCHAR *szTitle;
   char  *szCreationData;
 
   WORD  wCreateDataSize;
@@ -111,7 +108,7 @@ typedef struct {
 
 class CDialogTemplate {
 public:
-  CDialogTemplate(BYTE* pbData, unsigned int uCodePage=CP_ACP);
+  CDialogTemplate(BYTE* pbData, bool build_unicode, unsigned int uCodePage=CP_ACP);
   virtual ~CDialogTemplate();
 
   short GetWidth();
@@ -119,27 +116,33 @@ public:
   DialogItemTemplate* GetItem(WORD wId);
   DialogItemTemplate* GetItemByIdx(DWORD i);
   int   RemoveItem(WORD wId);
-  void  SetFont(char* szFaceName, WORD wFontSize);
+  void  SetFont(TCHAR* szFaceName, WORD wFontSize);
   void  AddItem(DialogItemTemplate item);
 #ifdef _WIN32
   HWND  CreateDummyDialog();
 #endif
   void  MoveAll(short x, short y);
   void  Resize(short x, short y);
-#ifdef _WIN32
   void  PixelsToDlgUnits(short& x, short& y);
   void  DlgUnitsToPixels(short& x, short& y);
-  SIZE  GetStringSize(WORD id, char *str);
-  void  RTrimToString(WORD id, char *str, int margins);
-  void  LTrimToString(WORD id, char *str, int margins);
-  void  CTrimToString(WORD id, char *str, int margins);
+#ifdef _WIN32
+  static inline bool SupportsDialogUnitComputation() { return true; }
+  void  PixelsToDlgUnits(SIZE& siz);
+  SIZE  GetStringSize(WORD id, TCHAR *str);
+  void  RTrimToString(WORD id, TCHAR *str, int margins);
+  void  LTrimToString(WORD id, TCHAR *str, int margins);
+  void  CTrimToString(WORD id, TCHAR *str, int margins);
+#else
+  static inline bool SupportsDialogUnitComputation() { return false; }
 #endif
   void  ConvertToRTL();
   BYTE* Save(DWORD& dwSize);
+  static void FreeSavedTemplate(BYTE*pDT) { delete [] pDT; }
   DWORD GetSize();
 
 private:
   bool  m_bExtended;
+  bool  m_build_unicode;
 
   DWORD m_dwHelpId; // Extended only
 
@@ -150,16 +153,16 @@ private:
   DWORD m_dwExtStyle;
   DWORD m_dwStyle;
 
-  char* m_szMenu;
-  char* m_szClass;
-  char* m_szTitle;
+  WINWCHAR *m_szMenu;
+  WINWCHAR *m_szClass;
+  WINWCHAR *m_szTitle;
 
    // Only if DS_FONT style is set
   short m_sFontSize;
   short m_sFontWeight; // Extended only
   BYTE  m_bItalic; // Extended only
   BYTE  m_bCharset; // Extended only
-  char* m_szFont;
+  WINWCHAR *m_szFont;
 
   // For (en/de)coding Unicode
   unsigned int m_uCodePage;
