@@ -1,9 +1,14 @@
 #include <cppunit/CompilerOutputter.h>
 #include <cppunit/extensions/TestFactoryRegistry.h>
 #include <cppunit/ui/text/TestRunner.h>
+#include "../util.h" // for NSISRT_*
+
+NSISRT_DEFINEGLOBALS();
 
 int main(int argc, char* argv[])
 {
+  if (!NSISRT_Initialize()) return 1;
+
   // Get the top level suite from the registry
   CppUnit::Test *suite = CppUnit::TestFactoryRegistry::getRegistry().makeTest();
 
@@ -19,4 +24,4 @@ int main(int argc, char* argv[])
 
   // Return error code 1 if the one of test failed.
   return wasSucessful ? 0 : 1;
-}
\ No newline at end of file
+}
