@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2008 Nullsoft and Contributors
+ * Copyright (C) 1999-2019 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -12,13 +12,17 @@
  * 
  * This software is provided 'as-is', without any express or implied
  * warranty.
+ *
+ * Reviewed for Unicode support by Jim Park -- 08/22/2007
  */
 
 #ifndef _EXEC_H_
 #define _EXEC_H_
 
-extern exec_flags g_exec_flags;
-extern exec_flags g_exec_flags_last_used;
+#include "api.h"
+
+extern exec_flags_t g_exec_flags;
+extern exec_flags_t g_exec_flags_last_used;
 
 int NSISCALL ExecuteCodeSegment(int pos, HWND hwndProgress); // returns 0 on success
 int NSISCALL ExecuteCallbackFunction(int num); // returns 0 on success
