@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2008 Nullsoft and Contributors
+ * Copyright (C) 1999-2019 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -12,6 +12,8 @@
  * 
  * This software is provided 'as-is', without any express or implied
  * warranty.
+ *
+ * Unicode support by Jim Park -- 08/13/2007
  */
 
 
@@ -21,7 +23,7 @@
 extern int *cur_langtable;
 
 extern int NSISCALL ui_doinstall(void);
-void NSISCALL update_status_text(int strtab, const char *text2);
+void NSISCALL update_status_text(int strtab, const TCHAR *text2);
 
 extern int ui_dlg_visible;
 extern HWND m_curwnd;
@@ -36,18 +38,12 @@ void NSISCALL build_g_logfile(void);
 // sent to every child window to tell it it can start executing NSIS code
 #define WM_NOTIFY_START (WM_USER+0x5)
 
-// sent to the outer window to tell it to go to the next inner window
-#define WM_NOTIFY_OUTER_NEXT (WM_USER+0x8)
-
 // sent to every child window to tell it it is closing soon
 #define WM_NOTIFY_INIGO_MONTOYA (WM_USER+0xb)
 
 // update message used by DirProc and SelProc for space display
 #define WM_IN_UPDATEMSG (WM_USER+0xf)
 
-// custom pages should send this message to let NSIS know they're ready
-#define WM_NOTIFY_CUSTOM_READY (WM_USER+0xd)
-
 // simulates clicking on the tree
 #define WM_TREEVIEW_KEYHACK (WM_USER+0x13)
 
