@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2008 Nullsoft and Contributors
+ * Copyright (C) 1999-2019 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,6 +14,8 @@
  * warranty.
  */
 
+/* Unicode support by Jim Park -- 11/16/2007 */
+
 #ifndef _ICON_H_
 #define _ICON_H_
 
@@ -24,7 +26,7 @@
 typedef struct
 {
   WORD wReserved;
-  WORD wIsIcon;
+  WORD wType;
   WORD wCount;
 } IconGroupHeader;
 
@@ -60,11 +62,11 @@ typedef struct
 
 typedef std::vector<Icon> IconGroup;
 
-IconGroup load_icon_file(const char* filename);
+IconGroup load_icon_file(const TCHAR* filename);
 IconGroup load_icon_res(CResourceEditor* re, WORD id);
 void free_loaded_icon(IconGroup icon);
 
-void set_icon(CResourceEditor* re, WORD wIconId, IconGroup icon1, IconGroup icon2);
+void set_main_icon(CResourceEditor* re, WORD wIconId, IconGroup icon1, IconGroup icon2);
 
 #ifdef NSIS_CONFIG_UNINSTALL_SUPPORT
 // returns the data of the uninstaller icon (inside filename) that should replace the installer icon data
