@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2008 Nullsoft and Contributors
+ * Copyright (C) 1999-2019 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,9 +15,10 @@
  */
 
 #include "mmap.h"
-
 #include <cstdio> // for f*
 #include <cassert> // for assert
+#include "tchar.h"
+#include <limits.h>
 #ifndef _WIN32
 #  include <sys/types.h> // for freebsd
 #  include <sys/mman.h>
@@ -25,6 +26,7 @@
 #  include <fcntl.h>
 #  include <unistd.h>
 #endif
+#include "util.h"
 
 // ========
 // MMapFile
@@ -146,10 +148,10 @@ void MMapFile::resize(int newsize)
 #ifdef _WIN32
     if (m_hFile == INVALID_HANDLE_VALUE)
     {
-      char buf[MAX_PATH], buf2[MAX_PATH];
+      TCHAR buf[MAX_PATH], buf2[MAX_PATH];
 
       GetTempPath(MAX_PATH, buf);
-      GetTempFileName(buf, "nsd", 0, buf2);
+      GetTempFileName(buf, _T("nsd"), 0, buf2);
 
       m_hFile = CreateFile(
         buf2,
@@ -215,12 +217,10 @@ void MMapFile::resize(int newsize)
     if (m_hFileDesc == -1)
 #endif
     {
-      extern FILE *g_output;
       extern void quit(); extern int g_display_errors;
       if (g_display_errors)
       {
-        fprintf(g_output,"\nInternal compiler error #12345: error creating mmap the size of %d.\n", m_iSize);
-        fflush(g_output);
+        PrintColorFmtMsg_ERR(_T("\nInternal compiler error #12345: error creating mmap the size of %d.\n"), m_iSize);
       }
       quit();
     }
@@ -248,12 +248,10 @@ void *MMapFile::get(int offset, int *sizep) const
 
   if (!m_iSize || offset + size > m_iSize)
   {
-    extern FILE *g_output;
     extern void quit(); extern int g_display_errors;
     if (g_display_errors) 
     {
-      fprintf(g_output,"\nInternal compiler error #12345: error mmapping file (%d, %d) is out of range.\n", offset, size);
-      fflush(g_output);
+      PrintColorFmtMsg_ERR(_T("\nInternal compiler error #12345: error mmapping file (%d, %d) is out of range.\n"), offset, size);
     }
     quit();
   }
@@ -263,12 +261,10 @@ void *MMapFile::get(int offset, int *sizep) const
   size += offset - alignedoffset;
 
 #ifdef _WIN32
-  const_cast<MMapFile*>(this)->m_pView =
-    MapViewOfFile(m_hFileMap, m_bReadOnly ? FILE_MAP_READ : FILE_MAP_WRITE, 0, alignedoffset, size);
+  m_pView = MapViewOfFile(m_hFileMap, m_bReadOnly ? FILE_MAP_READ : FILE_MAP_WRITE, 0, alignedoffset, size);
 #else
-  const_cast<MMapFile*>(this)->m_pView =
-    mmap(0, size, m_bReadOnly ? PROT_READ : PROT_READ | PROT_WRITE, MAP_SHARED, m_hFileDesc, alignedoffset);
-  const_cast<MMapFile*>(this)->m_iMappedSize = *sizep = size;
+  m_pView = mmap(0, size, m_bReadOnly ? PROT_READ : PROT_READ | PROT_WRITE, MAP_SHARED, m_hFileDesc, alignedoffset);
+  m_iMappedSize = *sizep = size;
 #endif
 
 #ifdef _WIN32
@@ -277,12 +273,10 @@ void *MMapFile::get(int offset, int *sizep) const
   if (m_pView == MAP_FAILED)
 #endif
   {
-    extern FILE *g_output;
     extern void quit(); extern int g_display_errors;
     if (g_display_errors) 
     {
-      fprintf(g_output,"\nInternal compiler error #12345: error mmapping datablock to %d.\n", size);
-      fflush(g_output);
+      PrintColorFmtMsg_ERR(_T("\nInternal compiler error #12345: error mmapping datablock to %d.\n"), size);
     }
     quit();
   }
@@ -297,11 +291,11 @@ void *MMapFile::getmore(int offset, int size) const
 #ifndef _WIN32
   int iMappedSizeBackup = m_iMappedSize;
 #endif
-  const_cast<MMapFile*>(this)->m_pView = 0;
+  m_pView = 0;
   pView = get(offset, size);
-  const_cast<MMapFile*>(this)->m_pView = pViewBackup;
+  m_pView = pViewBackup;
 #ifndef _WIN32
-  const_cast<MMapFile*>(this)->m_iMappedSize = iMappedSizeBackup;
+  m_iMappedSize = iMappedSizeBackup;
 #endif
   return pView;
 }
@@ -314,7 +308,7 @@ void MMapFile::release()
 #ifdef _WIN32
   UnmapViewOfFile(m_pView);
 #else
-  munmap(m_pView, m_iMappedSize);
+  munmap((char *)m_pView, m_iMappedSize);
 #endif
   m_pView = NULL;
 }
@@ -324,13 +318,13 @@ void MMapFile::release(void *pView, int size)
   if (!pView)
     return;
 
-  unsigned int alignment = ((unsigned int)pView) % m_iAllocationGranularity;
+  unsigned int alignment = ((ULONG_PTR)pView) % m_iAllocationGranularity;
   pView = (char *)pView - alignment;
   size += alignment;
 #ifdef _WIN32
   UnmapViewOfFile(pView);
 #else
-  munmap(pView, size);
+  munmap((char *)pView, size);
 #endif
 }
 
@@ -338,9 +332,9 @@ void MMapFile::flush(int num)
 {
   if (m_pView)
 #ifdef _WIN32
-    FlushViewOfFile(m_pView, num);
+  {} // improving performance by commenting: FlushViewOfFile(m_pView, num);
 #else
-    msync(m_pView, num, MS_SYNC);
+    msync((char *)m_pView, num, MS_SYNC);
 #endif
 }
 
@@ -433,6 +427,8 @@ void MMapBuf::resize(int newlen)
   if (newlen > m_alloc)
   {
     m_alloc = newlen + (16 << 20); // add 16mb to top of mapping
+    if (m_alloc < 0) // we've hit a signed integer overflow
+        m_alloc = INT_MAX;
 
     m_fm.resize(m_alloc);
 
